/// <reference types="react" />
import { NodeResizerProps } from './types';
export default function NodeResizer({ nodeId, isVisible, handleClassName, handleStyle, lineClassName, lineStyle, color, minWidth, minHeight, maxWidth, maxHeight, keepAspectRatio, shouldResize, onResizeStart, onResize, onResizeEnd, }: NodeResizerProps): JSX.Element | null;
//# sourceMappingURL=NodeResizer.d.ts.map