type GetDirectionParams = {
    width: number;
    prevWidth: number;
    height: number;
    prevHeight: number;
    invertX: boolean;
    invertY: boolean;
};
export declare function getDirection({ width, prevWidth, height, prevHeight, invertX, invertY }: GetDirectionParams): number[];
export {};
//# sourceMappingURL=utils.d.ts.map